@@ -1,11 +1,12 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004,2006 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2006,
+   2007 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
   
    GNU Radius is distributed in the hope that it will be useful,
@@ -460,6 +461,8 @@ snmpserv_before_config_hook(void *a ARG_UNUSED, void *b ARG_UNUSED)
 static void
 snmpserv_after_config_hook(void *arg, void *data ARG_UNUSED)
 {
+	stat_done();
+	stat_init();
 	if (server_stat) {
 		grad_nas_t *nas;
 		grad_iterator_t *itr;
@@ -852,8 +855,8 @@ mib_get_next(struct mib_node_t *node, struct snmp_var **varp, int *errp)
         struct snmp_var *temp_var;
         struct mib_node_t *found_node;
         
-        GRAD_DEBUG(2, ("OID %s",
-                  sprint_oid(buf, sizeof(buf), (*varp)->name)));
+        GRAD_DEBUG1(2, "OID %s",
+                    sprint_oid(buf, sizeof(buf), (*varp)->name));
         
         /* first, find the node itself */
         rc = mib_lookup(node, oid, OIDLEN(oid), &found_node);
@@ -882,9 +885,9 @@ mib_get_next(struct mib_node_t *node, struct snmp_var **varp, int *errp)
                 if (!node) {
                         /* The subtree is exhausted. Roll back until we find
                            first non-traversed down link */
-                        GRAD_DEBUG(2, ("rolling back from %d:%d",
-                                  found_node->index,
-                                  found_node->subid));
+                        GRAD_DEBUG2(2, "rolling back from %d:%d",
+                                    found_node->index,
+                                    found_node->subid);
                         while (node = found_node->up) {
                                 mib_reset(node);
                                 if (node->down && node->down != found_node)
@@ -896,9 +899,9 @@ mib_get_next(struct mib_node_t *node, struct snmp_var **varp, int *errp)
                         }
 
                         if (node)
-                                GRAD_DEBUG(2, ("rollback stopped at %d:%d",
-                                          node->index,
-                                          node->subid));
+                                GRAD_DEBUG2(2, "rollback stopped at %d:%d",
+                                            node->index,
+                                            node->subid);
 
                         if (node && node->subid != SUBID_X)
                                 node = node->down;
@@ -918,9 +921,7 @@ mib_get_next(struct mib_node_t *node, struct snmp_var **varp, int *errp)
         temp_var = snmp_var_create(oid);
         snmp_free(oid);
 
-        GRAD_DEBUG(2, ("NXT %s",
-                  sprint_oid(buf, sizeof(buf),
-                             temp_var->name)));
+        GRAD_DEBUG1(2, "NXT %s", sprint_oid(buf, sizeof(buf), temp_var->name));
                          
         *varp = temp_var;
         (*found_node->handler)(MIB_NODE_GET,
@@ -1020,11 +1021,11 @@ snmp_agent_response(struct snmp_pdu *pdu, int access)
                         /* First, check for the consistency of
                          * the request (rfc1157, 4.1.5):
                          */
-                        GRAD_DEBUG(1, ("SetRequest-PDU"));
+                        GRAD_DEBUG(1, "SetRequest-PDU");
                         if (access == SNMP_RO) {
                                 answer->err_stat = SNMP_ERR_GENERR;
                                 answer->err_ind = 1;
-                                GRAD_DEBUG(1, ("bad access mode"));
+                                GRAD_DEBUG(1,"bad access mode");
                                 return answer;
                         }
                         for (vp = pdu->var; vp; vp = vp->next) {
@@ -1038,7 +1039,7 @@ snmp_agent_response(struct snmp_pdu *pdu, int access)
                         if (answer->err_stat != SNMP_ERR_NOERROR) {
                                 answer->var = snmp_var_dup_list(pdu->var);
                                 answer->err_ind = index;
-                                GRAD_DEBUG(1, ("returning error"));
+                                GRAD_DEBUG(1, "returning error");
                                 return answer;
                         }
 
@@ -1057,11 +1058,11 @@ snmp_agent_response(struct snmp_pdu *pdu, int access)
                                 vresp = &vnew->next;
                         }
 
-                        GRAD_DEBUG(1, ("success"));
+                        GRAD_DEBUG(1, "success");
                         return answer;
 
                 case SNMP_PDU_GET:
-                        GRAD_DEBUG(1, ("GetRequest-PDU"));
+                        GRAD_DEBUG(1, "GetRequest-PDU");
 
                         vresp = &answer->var;
                         /* Loop through all variables */
@@ -1078,7 +1079,7 @@ snmp_agent_response(struct snmp_pdu *pdu, int access)
                                 if (answer->err_stat != SNMP_ERR_NOERROR
                                     || vnew == NULL) {
                                         answer->err_ind = index;
-                                        GRAD_DEBUG(1, ("returning"));
+                                        GRAD_DEBUG(1,"returning");
                                         /* preserve the rest of vars */
                                         *vresp = snmp_var_dup_list(vp);
                                         return answer;
@@ -1093,7 +1094,7 @@ snmp_agent_response(struct snmp_pdu *pdu, int access)
                         return answer;
 
                 case SNMP_PDU_GETNEXT:
-                        GRAD_DEBUG(1, ("GetNextRequest-PDU"));
+                        GRAD_DEBUG(1, "GetNextRequest-PDU");
 
                         vresp = &answer->var;
                         /* Loop through all variables */
@@ -1108,8 +1109,9 @@ snmp_agent_response(struct snmp_pdu *pdu, int access)
                                 if (answer->err_stat != SNMP_ERR_NOERROR
                                     || vnew == NULL) {
                                         answer->err_ind = index;
-                                        GRAD_DEBUG(1, ("returning: err_stat=%d",
-						  answer->err_stat));
+                                        GRAD_DEBUG1(1, 
+                                                    "returning: err_stat=%d",
+						     answer->err_stat);
                                         /* preserve the rest of vars */
                                         *vresp = snmp_var_dup_list(vp);
                                         return answer;
@@ -2709,13 +2711,14 @@ snmp_decode(SNMP_REQ *req, u_char *buf, size_t len)
 }
 
 int
-snmp_req_decode(struct sockaddr_in *sa,
-		  void *input, size_t inputsize, void **output)
+snmp_req_decode(const struct sockaddr_in *srv_sa,
+		const struct sockaddr_in *clt_sa,
+		void *input, size_t inputsize, void **output)
 {
         SNMP_REQ *req;
 
 	req = grad_emalloc(sizeof *req);
-	req->addr = *sa;
+	req->addr = *clt_sa;
         if (snmp_decode(req, input, inputsize)) {
                 grad_free(req);
                 return 1;
